#ifndef _GRAPHIC_H_
#define _GRAPHIC_H_

#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <graphics.h>
#include "objects.h"

#define NUMBER     0
#define OPERATION  1
#define ARGUMENT   2
#define FUNCTION   3
#define END        4

#define OK     0
#define ERROR  -1

inline void error(char *s)
{
  Draw3DBox(200, 100, 240, 90);
  setcolor(0);
  settextstyle(SMALL_FONT, HORIZ_DIR, 5);
  outtextxy(220, 110, "Error: ");
  outtextxy(220, 130, s);
  outtextxy(220, 150, "Press any key...");
  settextstyle(SMALL_FONT, HORIZ_DIR, 4);
  getch();
}

class TokenString
{
    char *ptr;
    char str[81];
  public:
    TokenString(char *s) { strcpy(ptr = str, s); }
    void reset() { ptr = str; }
    void gettoken();
    int type;
    char t[81];
    double val;
};

void TokenString::gettoken()
{
  while (isspace(*ptr)) ptr++;
  if (!*ptr)
  {
    *t = 0;
    type = END;
  }
  else if (strchr("+-*/()", *ptr))
  {
    t[0] = *ptr++;
    t[1] = 0;
    type = OPERATION;
  }
  else if (toupper(*ptr) == 'X')
  {
    t[0] = *ptr++;
    t[1] = 0;
    type = ARGUMENT;
  }
  else if (isdigit(*ptr))
  {
    int p = 0;
    char *temp = t;
    while (isdigit(*ptr) || (*ptr == '.' && !p))
    {
      if (*ptr == '.') p = 1;
      *temp++ = *ptr++;
    }
    *temp = 0;
    val = atof(t);
    type = NUMBER;
  }
  else if (isalpha(*ptr))
  {
    char *temp = t;
    while (isalpha(*ptr) || isdigit(*ptr)) *temp++ = *ptr++;
    *temp = 0;
    type = FUNCTION;
  }
  else
  {
    char s[41];
    sprintf(s, "Bad symvol '%c' %x \n", *ptr, *ptr);
    error(s);
  }
}

class Function : private TokenString
{
    double x;
    void primitive(double&);
    void level1(double&);
    void level2(double&);
    void level3(double&);
    void level4(double&);
    void level5(double&);
  public:
    int status;
    Function(char *f);
    int empty;
    double get(double);
};

Function::Function(char *f): TokenString(f)
{
  status = OK;
}

double Function::get(double a)
{
  x = a;
  reset();
  gettoken();
  if (type == END)
  {
    empty = 1;
    return 0;
    //error("No function");
  }
  else empty = 0;
  double result;
  level1(result);
  return result;
}

void Function::level1(double &result)
{
  double temp;
  char op;
  level2(result);
  while ((op = *t) == '+' || op == '-')
  {
    gettoken();
    level2(temp);
    if (!status)
    {
      if (op == '+')
        result += temp;
      else
        result -= temp;
    }
  }
}

void Function::level2(double &result)
{
  double temp;
  char op;
  level3(result);
  while((op = *t) == '*' || op == '/')
  {
    gettoken();
    level3(temp);
    if (!status)
    {
      if (op == '*')
        result *= temp;
      else
        result /= temp;
    }
  }
}

void Function::level3(double &result)
{
  int f = 0;
  char str[81];
  if (type == FUNCTION)
  {
    f = 1;
    strcpy(str, t);
    gettoken();
  }
  level4(result);
  if (f)
  {
    strupr(str);
    if (!strcmp("SIN", str))
      result = sin(result);
    else if (!strcmp("COS", str))
      result = cos(result);
    else if (!strcmp("TG", str))
      result = sin(result) / cos(result);
    else if (!strcmp("ABS", str))
      result = fabs(result);
    else if (!strcmp("LN", str))
    {
      if (result != 0)
        result = log(result);
    }
    else if (!strcmp("SQR", str))
      result *= result;
    else if (!strcmp("RND", str))
      result = random(result);
    else if (!strcmp("QUAD", str))
    {
      if (sin(result) > 0)
        result = 1;
      else
        result = 0;
    }
    else
    {
      error("Unknown function");
      status = ERROR;
      result = -1;
    }
  }
}

void Function::level4(double &result)
{
  int sign = 0;
  if (*t == '-')
  {
    sign = 1;
    gettoken();
  }
  level5(result);
  if (sign) result = -result;
}

void Function::level5(double &result)
{
  if (*t == '(')
  {
    gettoken();
    level1(result);
    if (*t != ')')
    {
      error("No ')'");
      status = ERROR;
    }
    gettoken();
  }
  else primitive(result);
}

void Function::primitive(double &result)
{
  switch (type)
  {
    case NUMBER:
      result = val;
      gettoken();
      break;
    case ARGUMENT:
      result = x;
      gettoken();
      break;
    default: status = ERROR;// error("Number or X needed");
  }
}

#endif

