#ifndef _MOUSE_H_
#define _MOUSE_H_
#include <dos.h>

int mouse_init();
void mouse_show();
void mouse_hide();
void mouse_read(int &x, int &y, int &buttons);
void mouse_set_pos(int x, int y);
void mouse_read_on_down(int &x, int &y, int button);
void mouse_read_on_up(int &x, int &y, int button);
void set_mouse_range(int xmin, int xmax, int ymin, int ymax);
void set_exception_region(int x1, int y1, int x2, int y2);

int mouse_init()
{
  struct REGPACK reg;
  reg.r_ax = 0;
  intr(0x33, &reg);
  if (reg.r_ax == 0xFFFF) return 1;
  return 0;
}

void mouse_show()
{
  struct REGPACK reg;
  reg.r_ax = 1;
  intr(0x33, &reg);
}

void mouse_hide()
{
  struct REGPACK reg;
  reg.r_ax = 2;
  intr(0x33, &reg);
}

void mouse_read(int &x, int &y, int &buttons)
{
  struct REGPACK reg;
  reg.r_ax = 3;
  intr(0x33, &reg);
  x = reg.r_cx;
  y = reg.r_dx;
  buttons = reg.r_bx;
}

void mouse_set_pos(int x, int y)
{
  struct REGPACK reg;
  reg.r_ax = 4;
  reg.r_cx = x;
  reg.r_dx = y;
  intr(0x33, &reg);
}

void mouse_read_on_down(int &x, int &y, int button)
{
  struct REGPACK reg;
  reg.r_ax = 5;
  reg.r_bx = button;
  intr(0x33, &reg);
  x = reg.r_cx;
  y = reg.r_dx;
}

void mouse_read_on_up(int &x, int &y, int button)
{
  struct REGPACK reg;
  reg.r_ax = 6;
  reg.r_bx = button;
  intr(0x33, &reg);
  x = reg.r_cx;
  y = reg.r_dx;
}

void set_mouse_range(int xmin, int xmax, int ymin, int ymax)
{
  struct REGPACK reg;
  reg.r_ax = 7;
  reg.r_cx = xmin;
  reg.r_dx = xmax;
  intr(0x33, &reg);
  reg.r_ax = 8;
  reg.r_cx = ymin;
  reg.r_dx = ymax;
  intr(0x33, &reg);
}

void set_exception_region(int x1, int y1, int x2, int y2)
{
  struct REGPACK reg;
  reg.r_ax = 0x10;
  reg.r_cx = x1;
  reg.r_dx = y1;
  reg.r_si = x2;
  reg.r_di = y2;
  intr(0x33, &reg);
}

#endif
