#ifndef _OBJECTS_H_
#define _OBJECTS_H_
#include <graphics.h>
#include <string.h>
#include <ctype.h>

#define chars "0123456789 abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-*/()."

struct TEvent {
  int mouse_x, mouse_y, buttons;
  unsigned int command;
  char key;
};

void Draw3DBox(int x, int y, int w, int h)
{
  setcolor(7);
  line(x, y, x + w - 1, y);
  line(x, y, x, y + h - 1);
  setcolor(15);
  line(x + 1, y + 1, x + 1, y + h - 3);
  line(x + 1, y + 1, x + w - 3, y + 1);
  setcolor(0);
  line(x, y + h - 1, x + w - 1, y + h - 1);
  line(x + w - 1, y, x + w - 1, y + h - 1);
  setcolor(8);
  line(x + 1, y + h - 2, x + w - 2, y + h - 2);
  line(x + w - 2, y + 1, x + w - 2, y + h - 2);
  setfillstyle(1, 7);
  bar(x + 2, y + 2, x + w - 3, y + h - 3);
}

void Draw3DHole(int x, int y, int w, int h, int color)
{
  setcolor(8);
  line(x, y, x + w - 2, y);
  line(x, y, x, y + h - 2);
  setcolor(0);
  line(x + 1, y + 1, x + w - 3, y + 1);
  line(x + 1, y + 1, x + 1, y + h - 3);
  setcolor(15);
  line(x + w - 1, y + h - 1, x, y + h - 1);
  line(x + w - 1, y + h - 1, x + w - 1, y);
  setcolor(7);
  line(x + w - 2, y + h - 2, x + 1, y + h - 2);
  line(x + w - 2, y + h - 2, x + w - 2, y + 1);
  setfillstyle(1, color);
  bar(x + 2, y + 2, x + w - 3, y + h - 3);
}

void Draw3DFrame(int x, int y, int w, int h, char *text)
{
  setcolor(15);
  rectangle(x + 1, y + 1, x + w - 1, y + h - 1);
  setcolor(8);
  rectangle(x, y, x + w - 2, y + h - 2);
  setcolor(0);
  putpixel(x, y + h - 1, 15);
  putpixel(x + w - 1, y, 15);

  settextstyle(DEFAULT_FONT, HORIZ_DIR, 1);
  setfillstyle(1, 7);
  bar(x + 10, y - 5, x + textwidth(text) + 17, y + 10);
  outtextxy(x + 15, y - 3, text);
}

class TObject {
  public:
    int x;
    int y;
    int w;
    int h;
    int command;
    int flags, old_flags;
    TObject *next;
    TObject(int x, int y, int w, int h, int command);
    ~TObject();
    virtual void HandleEvent(TEvent &Event, int &tag, int n);
    virtual void Draw();
};

TObject::TObject(int x, int y, int w, int h, int command)
{
  TObject::x = x;
  TObject::y = y;
  TObject::w = w;
  TObject::h = h;
  TObject::command = command;
  flags = 0;
  old_flags = 0;
  next = NULL;
}

TObject::~TObject()
{
}

void TObject::HandleEvent(TEvent &Event, int &tag, int n)
{
  if ((flags & 2) == 2)
    tag = n;
  if (flags != old_flags)
  {
    mouse_hide();
    Draw();
    mouse_show();
  }
  old_flags = flags;
  Event = Event;
}

void TObject::Draw()
{
}

class TButton: public TObject {
  private:
    char *caption;
    bool canselect;
    int old_event;
  public:
    TButton(int x, int y, int w, int h, int command, char *caption, bool canselect);
    ~TButton();
    virtual void HandleEvent(TEvent &Event, int &tag, int n);
    virtual void Draw();
};

TButton::TButton(int x, int y, int w, int h, int command, char *caption, bool canselect): TObject(x, y, w, h, command)
{
  TButton::caption = new char[strlen(caption + 1)];
  TButton::canselect = canselect;
  old_event = 0;
  strcpy(TButton::caption, caption);
  Draw();
}

TButton::~TButton()
{
  delete caption;
}

void TButton::HandleEvent(TEvent &Event, int &tag, int n)
{
  if ((tag != n) || (!canselect))
    flags &= 253;
  else
  {
    if ((flags & 4) == 4)
      flags &= 251;
    flags |= 2;
    if (Event.key == 32) flags |= 4;
  }
  if (Event.mouse_x >= x && Event.mouse_y >= y &&
      Event.mouse_x < x + w && Event.mouse_y < y + h)
  {
    if ((Event.buttons == 1) && (old_event == 0))
    {
      flags |= 3;
    }
    else
    {
      if ((flags & 3) == 3)
        flags |= 4;
      flags &= 254;
    }
    if (Event.buttons == 0) old_event = 0;
  }
  else
  {
    flags &= 254;
    old_event = Event.buttons;
  }

  TObject::HandleEvent(Event, tag, n);
  if ((flags & 4) == 4) Event.command = command;
}

void TButton::Draw()
{
  settextstyle(SMALL_FONT, HORIZ_DIR, 4);
  int text_x = x + w / 2 - textwidth(caption) / 2, text_y = y + h / 2 - textheight(caption) / 2 - 2;

  if ((flags & 2) == 2)
    setcolor(0);
  else
    setcolor(7);

  rectangle(x, y, x + w - 1, y + h - 1);
  setfillstyle(1, 7);
  bar(x + 2, y + 2, x + w - 3, y + h - 3);

  if ((flags & 1) == 1)
  {
    setcolor(8);
    rectangle(x + 1, y + 1, x + w - 2, y + h - 2);
    text_x++;
    text_y++;
  }
  else
  {
    setcolor(15);
    line(x + 1, y + 1, x + w - 2, y + 1);
    line(x + 1, y + 1, x + 1, y + h - 2);
    setcolor(0);
    line(x + w - 1, y + 1, x + w - 1, y + h - 1);
    line(x + 1, y + h - 1, x + w - 1, y + h - 1);
    setcolor(8);
    line(x + 2, y + h - 2, x + w - 2, y + h - 2);
    line(x + w - 2, y + 2, x + w - 2, y + h - 2);
  }

  setcolor(0);
  outtextxy(text_x, text_y, caption);
}

class TEdit: public TObject {
    int maxlen;
  public:
    char *text;
    TEdit(int x, int y, int w, int h, int command, int maxlen, char *text);
    ~TEdit();
    virtual void HandleEvent(TEvent &Event, int &tag, int n);
    virtual void Draw();
};

TEdit::TEdit(int x, int y, int w, int h, int command, int maxlen, char *text): TObject(x, y, w, h, command)
{
  TEdit::maxlen = maxlen;
  TEdit::text = new char[maxlen + 1];
  strcpy(TEdit::text, text);
  Draw();
}

void TEdit::Draw()
{
  int text_coord = h / 2 - textheight(text) / 2 + 1;

  Draw3DHole(x, y, w, h, 3);
  setcolor(0);
  settextstyle(SMALL_FONT, HORIZ_DIR, 4);
  outtextxy(x + text_coord, y + text_coord - 3, text);

  if ((flags & 2) == 2)
  {
    int n = x + text_coord + textwidth(text);
    line(n, y + 3, n, y + h - 4);
  }

  flags &= 251;
}

void TEdit::HandleEvent(TEvent &Event, int &tag, int n)
{
  char temp[2];

  if (tag != n)
    flags &= 253;
  else
    flags |= 2;
  if (Event.mouse_x >= x && Event.mouse_y >= y &&
      Event.mouse_x < x + w && Event.mouse_y < y + h)
  if (Event.buttons == 1)
      flags |= 2;

  if (((flags & 2) == 2) && Event.key && (Event.key != 13))
  {
    if ((Event.key == 8) && (strlen(text) != 0))
    {
      text[strlen(text) - 1] = 0;
      flags |= 4;
    }
    else if ((strlen(text) <= maxlen) && strchr(chars, Event.key))
    {
      text[strlen(text) + 1] = 0;
      text[strlen(text)] = Event.key;
      flags |= 4;
    }
  }

  TObject::HandleEvent(Event, tag, n);
  if (((flags & 2) == 2) && (Event.key == 13)) Event.command = command;
}

TEdit::~TEdit()
{
  delete text;
}

class TScrollBar: public TObject {
    int pos, pos_;
    float min;
    float max;
    TButton *left, *right;
    int old_event;
  public:
    char *text;
    TScrollBar(int x, int y, int w, int h, float pos, int command, float min, float max);
    ~TScrollBar();
    float getvalue();
    virtual void HandleEvent(TEvent &Event, int &tag, int n);
    virtual void Draw();
};

TScrollBar::TScrollBar(int x, int y, int w, int h, float pos, int command, float min, float max): TObject(x, y, w, h, command)
{
  TScrollBar::min = min;
  TScrollBar::max = max;
  TScrollBar::pos = pos * (w - h * 3) / float(max - min);
  pos_ = 0;
  old_event = 0;
  left = new TButton(x, y, h, h, command + 1, "<", 0);
  right = new TButton(x + w - 1 - h, y, h, h, command + 2, ">", 0);
  Draw();
}

void TScrollBar::Draw()
{
  if (pos != pos_)
  {
    setfillstyle(1, 0);
    bar(x + h, y, x + w - 1 - h, y + h - 1);
    Draw3DBox(x + h + pos, y, h, h);
  }
  if ((flags & 4) == 4) delay(30);
  flags = 0;
}

TScrollBar::~TScrollBar()
{
  delete left;
  delete right;
}

void TScrollBar::HandleEvent(TEvent &Event, int &tag, int n)
{
  if (tag != n) flags &= 253;

  if (Event.mouse_x >= x + h && Event.mouse_y >= y &&
      Event.mouse_x < x + w - h && Event.mouse_y < y + h)
  {
    if ((Event.buttons == 1) && (old_event == 0))
    {
      flags |= 2;
      pos = Event.mouse_x - x - h - h / 2;
      Event.command = command;
    }
    if (Event.buttons == 0) old_event = 0;
  }
  else old_event = Event.buttons;

  if (tag == n)
  switch (Event.key)
  {
    case 77: pos++; flags |= 2; Event.command = command; break;
    case 75: pos--; flags |= 2; Event.command = command; break;
  }

  left -> HandleEvent(Event, tag, n);
  right -> HandleEvent(Event, tag, n);

  if ((left -> flags & 1) == 1) {pos--; flags |= 4; Event.command = command; }
  if ((right -> flags & 1) == 1) {pos++; flags |= 4; Event.command = command; }

  if (pos < 0) pos = 0;
  if (pos > w - 3 * h) pos = w - 3 * h;

  TObject::HandleEvent(Event, tag, n);
  pos_ = pos;
}

float TScrollBar::getvalue()
{
  return (max - min) / (w - h * 3) * pos;
}

class TCheckBox: public TObject {
  public:
    int old_event;
    char *caption;
    TCheckBox(int x, int y, int w, int command, char *caption, bool checked);
    ~TCheckBox();
    int getstate();
    virtual void HandleEvent(TEvent &Event, int &tag, int n);
    virtual void Draw();
};

TCheckBox::TCheckBox(int x, int y, int w, int command, char *caption, bool checked): TObject(x, y, w, w, command)
{
  TCheckBox::caption = new char[strlen(caption + 1)];
  flags |= checked;
  strcpy(TCheckBox::caption, caption);
  old_event = 0;
  Draw();
}

void TCheckBox::Draw()
{
  if ((flags & 2) == 2)
    Draw3DHole(x, y, w, w, 14);
  else
    Draw3DHole(x, y, w, w, 15);
  if ((flags & 1) == 1)
  {
    setcolor(0);
    line(x + 3, y + 3, x + w - 4, y + h - 4);
    line(x + w - 4, y + 3, x + 3, y + h - 4);
  }
  setcolor(0);
  settextstyle(SMALL_FONT, HORIZ_DIR, 4);
  outtextxy(x + w + 5, y + 1, caption);
}

void TCheckBox::HandleEvent(TEvent &Event, int &tag, int n)
{
  if (tag != n)
    flags &= 253;
  else
  {
    flags |= 2;
    if (Event.key == 32) flags ^= 1;
  }
  if (Event.mouse_x >= x && Event.mouse_y >= y &&
      Event.mouse_x < x + w && Event.mouse_y < y + h)
  {
    if (Event.buttons == 1)
    {
      flags |= 2;
      if (old_event == 0)
      {
        flags ^= 1;
        Event.command = command;
      }
    }
  }

  old_event = Event.buttons;

  TObject::HandleEvent(Event, tag, n);
}

TCheckBox::~TCheckBox()
{
  delete caption;
}

#endif
